package com.heyang;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang.StringUtils;

/**
 * Parse json string to tokens
 * @author Heyang
 *
 */
public class Lexer {
	private List<Token> tokens;

	public Lexer(String jsonTxt) {
		tokens = new ArrayList<Token>();

		String bundle = "";
		for (int i = 0; i < jsonTxt.length(); i++) {
			char c = jsonTxt.charAt(i);
			
			if(c==':') {
				char t=c;
				char p=t;
				c=p;
			}

			if (Character.isWhitespace(c)) {
				continue;
			} else if (c == '{') {
				tokens.add(new Token(c, Token.TYPE_OPEN_BRACE));
			} else if (c == '}') {
				if (StringUtils.isNotEmpty(bundle)) {
					tokens.add(new Token(bundle, Token.TYPE_TEXT));
					bundle = "";
				}

				tokens.add(new Token(c, Token.TYPE_CLOSE_BRACE));
			} else if (c == '[') {
				tokens.add(new Token(c, Token.TYPE_OPEN_BRACKET));
			} else if (c == ']') {
				if (StringUtils.isNotEmpty(bundle)) {
					tokens.add(new Token(bundle, Token.TYPE_TEXT));
					bundle = "";
				}
				
				tokens.add( new Token(c, Token.TYPE_CLOSE_BRACKET));
			} else if (c == ',') {
				if (StringUtils.isNotEmpty(bundle)) {
					tokens.add(new Token(bundle, Token.TYPE_TEXT));
					bundle = "";
				}

				tokens.add(new Token(c, Token.TYPE_COMMA));
			} else if (c == ':') {
				if (StringUtils.isNotEmpty(bundle)) {
					tokens.add(new Token(bundle, Token.TYPE_TEXT));
					bundle = "";
				}

				tokens.add(new Token(c, Token.TYPE_COLON));
			}else if(c == '\"') {
				int idx=i+1;
				
				while(idx<jsonTxt.length()) {
					char cEnd = jsonTxt.charAt(idx);
					
					if (cEnd == '\"') {
						break;
					}
					
					idx++;
				}
				
				String sub=jsonTxt.substring(i, idx+1);
				tokens.add(new Token(sub, Token.TYPE_TEXT));
				i=idx;
			} else {
				bundle += c;
			}
		}
		
		setTokenIndexes();
	}

	public void setTokenIndexes() {
		int idx = 0;
		for (Token t : tokens) {
			idx++;
			t.setIndex(idx);
		}
	}

	public void printTokens() {
		int idx = 0;
		for (Token t : tokens) {
			idx++;
			t.setIndex(idx);
			System.out.println("#" + idx + " " + t.getText());
		}
	}
	
	public String getCompactJsonTxt() {
		StringBuilder sb=new StringBuilder();
		
		for (Token t : tokens) {
			sb.append(t.getText());
		}
		
		return sb.toString();
	}
	
	public List<Token> getTokenList() {
		return tokens;
	}
}
