package com.heyang;

import java.util.List;

public class TreeBuilder {
	private Node root;
	private List<Token> tokens;
	private int tokenIdx;
	
	public TreeBuilder(List<Token> tokens)  throws Exception{
		this.tokens=tokens;
		this.tokenIdx=0;
		
		root=new Node(null,Node.Type_List);
		parse_object(root);
	}
	
	private void parse_object(Node parent) throws Exception{
		Token token;
		
		token=fetchToken();
		if(token.getType()!=Token.TYPE_OPEN_BRACE) {
			throw new Exception("Expected:'{' actual:"+token.getText()+" "+token);
		}
		
		for(;;) {
			token=fetchToken();
			if(token.getType()!=Token.TYPE_TEXT) {
				returnToken();
				break;
			}
			String key=token.getText();
			
			token=fetchToken();
			if(token.getType()!=Token.TYPE_COLON) {
				throw new Exception("Expected:':' actual:"+token.getText()+" "+token);
			}
			
			token=fetchToken();
			if(token.getType()==Token.TYPE_TEXT) {
				String value=token.getText();
				parent.addChild(new Node(key,value));
			}else if(token.getType()==Token.TYPE_OPEN_BRACE){
				Node node=new Node(key,Node.Type_List);
				parent.addChild(node);
				returnToken();
				parse_object(node);
			}else if(token.getType()==Token.TYPE_OPEN_BRACKET) {
				Node node=new Node(key,Node.Type_Array);
				parse_array(node);
				parent.addChild(node);
			}else {
				throw new Exception("value should be string/object/array but not.");
			}
			
			token=fetchToken();
			if(token.getType()==Token.TYPE_COMMA) {
				continue;
			}else {
				returnToken();
				break;
			}
		}
		
		token=fetchToken();
		if(token.getType()!=Token.TYPE_CLOSE_BRACE) {
			throw new Exception("Expected:'}' actual:"+token.getText()+" "+token);
		}
	}
	
	private void parse_array(Node parent) throws Exception {
		Token token;
		
		for(;;) {
			token=fetchToken();
			if(token.getType()==Token.TYPE_TEXT) {
				String value=token.getText();
				Node node=new Node(null,value);
				parent.addChild(node);
			}else if(token.getType()==Token.TYPE_OPEN_BRACE) {
				Node node=new Node(null,Node.Type_List);
				parent.addChild(node);
				
				returnToken();
				parse_object(node);
			}else {
				returnToken();
			}
			
			token=fetchToken();
			if(token.getType()==Token.TYPE_COMMA) {
				continue;
			}else {
				returnToken();
				break;
			}
		}
		
		token=fetchToken();
		if(token.getType()!=Token.TYPE_CLOSE_BRACKET) {
			throw new Exception("Expected:']' actual:"+token.getText()+" "+token);
		}
	}
	
	private Token fetchToken() {
		if(tokenIdx>=tokens.size()) {
			return null;
		}else {
			Token t=tokens.get(tokenIdx);
			tokenIdx++;
			return t;
		}		
	}
	
	private void returnToken() {
		if(tokenIdx>0) {
			tokenIdx--;
		}
	}
	
	public Node getRoot() {
		return root;
	}
}
